/**
 * ==============================================
 *  PMX : Performance Model eXtractor
 * ==============================================
 *
 * (c) Copyright 2014-2015, by Juergen Walter and Contributors.
 *
 * Project Info:   http://descartes.tools/pmx
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pmx.filter.resourcedemands;

import java.util.HashMap;
import java.util.List;
import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.common.configuration.Configuration;
import kieker.tools.traceAnalysis.filter.AbstractTraceAnalysisFilter;
import kieker.tools.traceAnalysis.systemModel.AbstractMessage;
import kieker.tools.traceAnalysis.systemModel.Execution;
import kieker.tools.traceAnalysis.systemModel.MessageTrace;

@Plugin(name = "ExternalCallTimesAppenderFilter", description = "Adds times for external calls to total time.", outputPorts = { @OutputPort(name = AbstractResourceDemandFilter.OUTPUT_PORT_NAME_DEMANDS, description = "Outputs resource demands", eventTypes = { HashMap.class }), })
public class ExternalCallTimesAppenderFilter extends AbstractTraceAnalysisFilter{

	public ExternalCallTimesAppenderFilter(final Configuration configuration,
			final IProjectContext projectContext) {
		super(configuration, projectContext);
	}

	public static final String INPUT_PORT_NAME_MESSAGE_TRACE = "messageTrace";

	@InputPort(name = ExternalCallTimesAppenderFilter.INPUT_PORT_NAME_MESSAGE_TRACE, description = "Extract class-information from monitoring record", eventTypes = { MessageTrace.class })
	public void runMessageTraceFilter(final MessageTrace trace) {
//		Map<Execution, Double> externalCallTime = new HashMap<Execution, Double>();
//		Map<String, Double> aggregatedExternalCallTime = new HashMap<String, Double>();
//		Map<String, Double> aggregatedCallTime = new HashMap<String, Double>();
//		Map<String, List<String>> externalCallMethods = new HashMap<String, List<String>>();

		List<AbstractMessage> messages = trace.getSequenceAsVector();
		for (AbstractMessage message : messages) {
			Execution x = message.getSendingExecution();
			Execution y = message.getReceivingExecution();

			if (x.getEss() < y.getEss()) {				
//				x.getTout = y.getTout // y always less
			}
		}	
	}

	@Override
	public void terminate(boolean errorBeforeTermination) {
		super.terminate(errorBeforeTermination);
	}

}
