/**
 * ==============================================
 *  PMX : Performance Model eXtractor
 * ==============================================
 *
 * (c) Copyright 2014-2015, by Juergen Walter and Contributors.
 *
 * Project Info:   http://descartes.tools/pmx
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pmx.filter.resourcedemands;

import java.io.File;
import java.util.concurrent.TimeUnit;

import kieker.analysis.IAnalysisController;
import kieker.analysis.exception.AnalysisConfigurationException;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.tools.traceAnalysis.filter.AbstractMessageTraceProcessingFilter;
import kieker.tools.traceAnalysis.filter.AbstractTraceAnalysisFilter;
import kieker.tools.traceAnalysis.filter.flow.TraceEventRecords2ExecutionAndMessageTraceFilter;
import kieker.tools.traceAnalysis.filter.traceReconstruction.TraceReconstructionFilter;
import kieker.tools.traceAnalysis.filter.visualization.dependencyGraph.OperationDependencyGraphAllocationFilter;
import kieker.tools.traceAnalysis.filter.visualization.dependencyGraph.ResponseTimeNodeDecorator;
import kieker.tools.traceAnalysis.systemModel.repository.SystemModelRepository;
import tools.descartes.pmx.filter.WorkloadFilter;
import tools.descartes.pmx.filter.controlflow.CallNodeDecorator;
import tools.descartes.pmx.filter.resourcedemands.CPUFilter;

public class ResourceDemandFilterAppender {

	private static InternalResourceDemandFilter addInternalResourceDemandFilter(
			final AbstractFilterPlugin traceEvents2ExecutionAndMessageTraceFilter,
			CPUFilter cpuFilter, String resultPath,
			IAnalysisController analysisController)
			throws AnalysisConfigurationException {

		final Configuration resourceDemandFilterConfiguration = new Configuration();
		resourceDemandFilterConfiguration.setProperty(
				AbstractResourceDemandFilter.CONFIG_PROPERTY_NAME_OUTPUT_FN,
				resultPath + File.separator+ "resourcedemands");
		(new File(resultPath + File.separator + "resourcedemands"+File.separator)).mkdirs();

		InternalResourceDemandFilter internalResourceDemandFilter = new InternalResourceDemandFilter(
				resourceDemandFilterConfiguration, analysisController);

		analysisController.connect(cpuFilter,
				CPUFilter.OUTPUT_PORT_NAME_UTILIZATION,
				internalResourceDemandFilter,
				InternalResourceDemandFilter.INPUT_PORT_NAME_UTILIZATION);

		analysisController
				.connect(
						traceEvents2ExecutionAndMessageTraceFilter,
						TraceEventRecords2ExecutionAndMessageTraceFilter.OUTPUT_PORT_NAME_MESSAGE_TRACE,
						internalResourceDemandFilter,
						AbstractMessageTraceProcessingFilter.INPUT_PORT_NAME_MESSAGE_TRACES);
		return internalResourceDemandFilter;
	}

	private static WorkloadFilter addWorkloadFilter(
			final AbstractFilterPlugin traceEvents2ExecutionAndMessageTraceFilter,
			IAnalysisController analysisController)
			throws AnalysisConfigurationException {

		WorkloadFilter workloadFilter = new WorkloadFilter(new Configuration(),
				analysisController);
		
		analysisController
		.connect(
				traceEvents2ExecutionAndMessageTraceFilter,
				TraceEventRecords2ExecutionAndMessageTraceFilter.OUTPUT_PORT_NAME_MESSAGE_TRACE,
				workloadFilter,
				AbstractMessageTraceProcessingFilter.INPUT_PORT_NAME_MESSAGE_TRACES);
		
		return workloadFilter;
	}
	
	/**
	 * Requires message traces
	 * 
	 * @param analysisController
	 * @param systemModelRepository
	 * @param traceEvents2ExecutionAndMessageTraceFilter
	 * @throws IllegalStateException
	 * @throws AnalysisConfigurationException
	 */
	private static OperationDependencyGraphAllocationFilter addOperationDependencyGraphAllocationFilter(
			IAnalysisController analysisController,
			SystemModelRepository systemModelRepository,
			AbstractFilterPlugin traceEvents2ExecutionAndMessageTraceFilter)
			throws IllegalStateException, AnalysisConfigurationException {
		final OperationDependencyGraphAllocationFilter operationDependencyGraphAllocationFilter = new OperationDependencyGraphAllocationFilter(
				new Configuration(), analysisController);
		operationDependencyGraphAllocationFilter
				.addDecorator(new ResponseTimeNodeDecorator(
						TimeUnit.NANOSECONDS));
		operationDependencyGraphAllocationFilter
				.addDecorator(new CallNodeDecorator());

		analysisController.connect(operationDependencyGraphAllocationFilter,
				AbstractTraceAnalysisFilter.REPOSITORY_PORT_NAME_SYSTEM_MODEL,
				systemModelRepository);
		analysisController
				.connect(
						traceEvents2ExecutionAndMessageTraceFilter,
						TraceEventRecords2ExecutionAndMessageTraceFilter.OUTPUT_PORT_NAME_MESSAGE_TRACE,
						operationDependencyGraphAllocationFilter,
						AbstractMessageTraceProcessingFilter.INPUT_PORT_NAME_MESSAGE_TRACES);
		return operationDependencyGraphAllocationFilter;
	}

}
